@@ -36,9 +36,11 @@
 ;;; Autoloads and Requires
 
 (ess-message "[ess-r-d:] (require 'ess-s-l)")
+(require 'cl)
 (require 'ess-s-l)
 (require 'eldoc)
-(require 'ess-developer)
+(require 'ess-utils)
+(require 'ess-r-package)
 (require 'ess-help)
 (require 'ess-roxy)
 (require 'ess-tracebug)
@@ -52,20 +54,20 @@
 (autoload 'ess-help-underline "ess-help" "(Autoload)" t)
 (autoload 'ess--flush-help-into-current-buffer "ess-help" "(Autoload)" t)
 
+
+;;*;; Mode definition
+
+;;;*;;; UI (Keymaps / Menus)
+
 (defvar ess-dev-map
   (let (ess-dev-map)
     (define-prefix-command 'ess-dev-map)
     ;; Note: some of these comand are automatically redefined by those in
-    (define-key ess-dev-map "\C-t" 'ess-toggle-developer)
-    (define-key ess-dev-map "t" 'ess-toggle-developer)
-    ;; (define-key ess-dev-map "\C-T" 'ess-toggle-tracebug)
+    (define-key ess-dev-map "\C-s" 'ess-r-set-evaluation-env)
+    (define-key ess-dev-map "s" 'ess-r-set-evaluation-env)
     (define-key ess-dev-map "T" 'ess-toggle-tracebug)
-    (define-key ess-dev-map "\C-a" 'ess-developer-add-package)
-    (define-key ess-dev-map "a" 'ess-developer-add-package)
-    (define-key ess-dev-map "\C-r" 'ess-developer-remove-package)
-    (define-key ess-dev-map "r" 'ess-developer-remove-package)
-    (define-key ess-dev-map "\C-l" 'ess-developer-load-package)
-    (define-key ess-dev-map "l" 'ess-developer-load-package)
+    (define-key ess-dev-map "\C-l" 'ess-r-devtools-load-package)
+    (define-key ess-dev-map "l" 'ess-r-devtools-load-package)
     (define-key ess-dev-map "`" 'ess-show-traceback)
     (define-key ess-dev-map "~" 'ess-show-call-stack)
     (define-key ess-dev-map "\C-w" 'ess-watch)
@@ -109,6 +111,27 @@
     ess-dev-map)
   "Keymap for commands related to development and debugging.")
 
+(defvar ess-r-package-dev-map
+  (let (ess-r-package-dev-map)
+    (define-prefix-command 'ess-r-package-dev-map)
+    (define-key ess-r-package-dev-map "\C-s" 'ess-r-package-set-package)
+    (define-key ess-r-package-dev-map "s"    'ess-r-package-set-package)
+    (define-key ess-r-package-dev-map "\C-c" 'ess-r-devtools-check-package)
+    (define-key ess-r-package-dev-map "c"    'ess-r-devtools-check-package)
+    (define-key ess-r-package-dev-map "\C-d" 'ess-r-devtools-document-package)
+    (define-key ess-r-package-dev-map "d"    'ess-r-devtools-document-package)
+    (define-key ess-r-package-dev-map "\C-i" 'ess-r-devtools-install-package)
+    (define-key ess-r-package-dev-map "i"    'ess-r-devtools-install-package)
+    (define-key ess-r-package-dev-map "\C-l" 'ess-r-devtools-load-package)
+    (define-key ess-r-package-dev-map "l"    'ess-r-devtools-load-package)
+    (define-key ess-r-package-dev-map "\C-r" 'ess-r-devtools-revdep-check-package)
+    (define-key ess-r-package-dev-map "r"    'ess-r-devtools-revdep-check-package)
+    (define-key ess-r-package-dev-map "\C-t" 'ess-r-devtools-test-package)
+    (define-key ess-r-package-dev-map "t"    'ess-r-devtools-test-package)
+    (define-key ess-r-package-dev-map "\C-u" 'ess-r-devtools-unload-package)
+    (define-key ess-r-package-dev-map "u"    'ess-r-devtools-unload-package)
+    ess-r-package-dev-map))
+
 (easy-menu-define ess-roxygen-menu nil
   "Roxygen submenu."
   '("Roxygen"
@@ -150,21 +173,20 @@
     "-----"
     ["About" ess-tracebug-show-help t]))
 
-(easy-menu-define ess-developer-menu nil
-  "Developer submenu."
-  '("Developer"
+(easy-menu-define ess-r-package-menu nil
+  "Package Development submenu."
+  '("Package development"
     :visible (and ess-dialect (string-match "^R" ess-dialect))
-    ["Active?"          ess-toggle-developer
+    ["Active?" ess-r-package-mode
      :style toggle
-     :selected ess-developer]
-    ["Add package" ess-developer-add-package t]
-    ["Remove package" ess-developer-remove-package t]))
+     :selected ess-r-package-mode]
+    ["Select package for evaluation" ess-r-set-evaluation-env t]))
 
 (easy-menu-add-item ess-mode-menu nil ess-roxygen-menu "end-dev")
-(easy-menu-add-item ess-mode-menu nil ess-developer-menu "end-dev")
+(easy-menu-add-item ess-mode-menu nil ess-r-package-menu "end-dev")
 (easy-menu-add-item ess-mode-menu nil ess-tracebug-menu "end-dev")
 
-(easy-menu-add-item inferior-ess-mode-menu nil ess-developer-menu "end-dev")
+(easy-menu-add-item inferior-ess-mode-menu nil ess-r-package-menu "end-dev")
 (easy-menu-add-item inferior-ess-mode-menu nil ess-tracebug-menu "end-dev")
 
 
@@ -181,29 +203,23 @@
 
 (ess-message "[ess-r-d:] (autoload ..) & (def** ..)")
 
-;; (autoload 'inferior-ess "ess-inf" "Run an ESS process.")
-;; (autoload 'ess-mode     "ess-mode" "Edit an ESS process.")
 
-(defvar R-customize-alist
+(defvar ess-r-customize-alist
   (append
-   '((ess-local-customize-alist         . 'R-customize-alist)
-     (ess-eldoc-function                . 'ess-R-eldoc-function)
+   '((ess-local-customize-alist         . 'ess-r-customize-alist)
+     (ess-eldoc-function                . #'ess-R-eldoc-function)
      (ess-dialect                       . "R")
      (ess-suffix                        . "R")
      (ess-ac-sources                    . '(ac-source-R))
-     (ess-company-backends		. '((company-R-args company-R-objects)))
+     (ess-company-backends              . '((company-R-args company-R-objects :sorted)))
      (ess-build-tags-command            . "rtags('%s', recursive = TRUE, pattern = '\\\\.[RrSs](rw)?$',ofile = '%s')")
      (ess-traceback-command             . "local({cat(geterrmessage(), \"---------------------------------- \n\", fill=TRUE);try(traceback(), silent=TRUE)})\n")
      (ess-call-stack-command            . "traceback(1)\n")
-     (ess-eval-command                  . ".ess.eval(\"%s\", FALSE, FALSE, file=\"%f\")\n")
-     (ess-eval-visibly-command          . ".ess.eval(\"%s\", TRUE, TRUE, 300, file=\"%f\")\n")
-     (ess-eval-visibly-noecho-command   . ".ess.eval(\"%s\", FALSE, TRUE, 300, file=\"%f\")\n")
-     (ess-load-command                  . ".ess.source(\"%s\", FALSE, FALSE)\n")
-     (ess-load-visibly-command          . ".ess.source(\"%s\", TRUE, TRUE, 300)\n")
-     (ess-load-visibly-noecho-command   . ".ess.source(\"%s\", FALSE, TRUE, 300)\n")
+     (ess-build-eval-message-function  . #'ess-r-build-eval-message)
      (ess-dump-filename-template        . (ess-replace-regexp-in-string
                                            "S$" ess-suffix ; in the one from custom:
                                            ess-dump-filename-template-proto))
+     (ess-build-help-command-function   . #'ess-r-build-help-command)
      (ess-help-web-search-command       . 'ess-R-sos)
      (ess-mode-syntax-table             . R-syntax-table)
      (ess-mode-editing-alist            . R-editing-alist)
@@ -225,13 +241,13 @@
      (inferior-ess-exit-command         . "q()")
      (inferior-ess-exit-prompt          . "Save workspace image? [y/n/c]: ")
      ;;harmful for shell-mode's C-a: -- but "necessary" for ESS-help?
-     (inferior-ess-start-file		. nil) ;; "~/.ess-R"
-     (inferior-ess-start-args		. "")
-     (ess-error-regexp-alist		. ess-R-error-regexp-alist)
+     (inferior-ess-start-file	          . nil) ;; "~/.ess-R"
+     (inferior-ess-start-args           . "")
+     (ess-error-regexp-alist            . ess-R-error-regexp-alist)
      (ess-describe-object-at-point-commands . 'ess-R-describe-object-at-point-commands)
-     (ess-STERM		. "iESS")
-     (ess-editor	. R-editor)
-     (ess-pager		. R-pager)
+     (ess-STERM                         . "iESS")
+     (ess-editor                        . R-editor)
+     (ess-pager                         . R-pager)
      (prettify-symbols-alist            . '(("<-" . ?←)
                                             ("<<-" . ?↞)
                                             ("->" . ?→)
@@ -239,6 +255,7 @@
    S-common-cust-alist)
   "Variables to customize for R -- set up later than emacs initialization.")
 
+(defalias 'R-customize-alist 'ess-r-customize-alist)
 
 (defvar R-editing-alist
   ;; copy the S-alist and modify :
@@ -256,7 +273,7 @@
 
 ;; takes precidence over R1 below in english locales, and allows spaces in file path
 (add-to-list 'compilation-error-regexp-alist-alist
-             '(R "\\(at \\(.+\\)[#:]\\([0-9]+\\)\\)"  2 3 nil 2 1))
+             '(R "\\(\\(?: at \\|(@\\)\\([^#\n]+\\)[#:]\\([0-9]+\\)\\)"  2 3 nil 2 1))
 
 (add-to-list 'compilation-error-regexp-alist-alist
              '(R1 " \\([^ \t\n]+\\)#\\([0-9]+\\)[: ]"  1 2 nil 2))
@@ -302,49 +319,19 @@ Set this variable to nil to disable searching for other versions of R.
 If you set this variable, you need to restart Emacs (and set this variable
 before ess-site is loaded) for it to take effect."))
 
+
+;;;*;;; Mode init
+
 (defvar ess-R-post-run-hook nil
   "Functions run in process buffer after the initialization of R
   process.")
 
-(defun ess--R-load-ESSR ()
-  "Load/INSTALL/Update ESSR."
-  (let* ((ESSR-directory (expand-file-name "ESSR" ess-etc-directory))
-         (src-dir (expand-file-name "R" ESSR-directory)))
-
-    (if (not (or (and (boundp 'ess-remote) ess-remote)
-                 (file-remote-p (ess-get-process-variable 'default-directory))))
-        (let ((cmd (format
-                    "local({
-                      source('%s/.load.R', local=TRUE) #define load.ESSR
-                      load.ESSR('%s')})\n"
-                    src-dir src-dir)))
-          (ess-write-to-dribble-buffer (format "load-ESSR cmd:\n%s\n" cmd))
-          (with-current-buffer (ess-command cmd)
-            (let ((msg (buffer-string)))
-              (when (> (length msg) 1)
-                (message (format "load ESSR: %s" msg))))))
-      ;; else, remote
-      (let* ((verfile (expand-file-name "VERSION" ESSR-directory))
-             (loadremote (expand-file-name "LOADREMOTE" ESSR-directory))
-             (version (if (file-exists-p verfile)
-                          (with-temp-buffer
-                            (insert-file-contents verfile)
-                            (buffer-string))
-                        (error "Cannot find ESSR source code")))
-             (r-load-code (with-temp-buffer
-                            (insert-file-contents loadremote)
-                            (buffer-string))))
-        (ess-write-to-dribble-buffer (format "version file: %s\nloadremote file: %s\n"
-                                             verfile loadremote))
-        (unless (ess-boolean-command (format r-load-code version) nil 0.1)
-          (let ((errmsg (with-current-buffer " *ess-command-output*" (buffer-string)))
-                (files (directory-files src-dir t "\\.R$")))
-            (ess-write-to-dribble-buffer (format "error loading ESSR.rda: \n%s\n" errmsg))
-            ;; should not happen, unless extrem conditions (ancient R or failed download))
-            (message "Failed to download ESSR.rda (see *ESS* buffer). Injecting ESSR code from local machine")
-            (ess-command (format ".ess.ESSRversion <- '%s'\n" version)) ; cannot do this at R level
-            (mapc #'ess--inject-code-from-file files)))))))
+(defun ess-r-mode-p ()
+  "Check whether we have a buffer running in R mode.
 
+This is to get around the lack of proper derived modes in ESS."
+  (and (eq major-mode 'ess-mode)
+       (string= ess-dialect "R")))
 
 ;;;### autoload
 (defun R (&optional start-args)
@@ -352,7 +339,11 @@ before ess-site is loaded) for it to take effect."))
 Optional prefix (C-u) allows to set command line arguments, such as
 --vsize.  This should be OS agnostic.
 If you have certain command line arguments that should always be passed
-to R, put them in the variable `inferior-R-args'."
+to R, put them in the variable `inferior-R-args'.
+
+START-ARGS can be a string representing an argument, a list of
+such strings, or any other non-nil value. In the latter case, you
+will be prompted to enter arguments interactively."
   (interactive "P")
   (ess-write-to-dribble-buffer   ;; for debugging only
    (format
@@ -363,17 +354,24 @@ to R, put them in the variable `inferior-R-args'."
               "--ess "
             ;; else: "unix alike"
             (if (not ess-R-readline) "--no-readline ")))
+         (start-args
+          (cond ((stringp start-args)
+                 start-args)
+                ((and start-args
+                      (listp start-args)
+                      (every 'stringp start-args))
+                 (mapconcat 'identity start-args " "))
+                (start-args
+                 (read-string
+                  (concat "Starting Args"
+                          (if r-always-arg
+                              (concat " [other than '" r-always-arg "']"))
+                          " ? ")))))
          (r-start-args
           (concat r-always-arg
-                  inferior-R-args " " ; add space just in case
-                  (if start-args
-                      (read-string
-                       (concat "Starting Args"
-                               (if r-always-arg
-                                   (concat " [other than '" r-always-arg "']"))
-                               " ? "))
-                    nil)))
-         (cust-alist (copy-alist R-customize-alist))
+                  inferior-R-args " "   ; add space just in case
+                  start-args))
+         (cust-alist (copy-alist ess-r-customize-alist))
          (gdbp (string-match-p "gdb" r-start-args))
          use-dialog-box)
 
@@ -395,7 +393,7 @@ to R, put them in the variable `inferior-R-args'."
     (remove-hook 'completion-at-point-functions 'ess-filename-completion 'local) ;; should be first
     (add-hook 'completion-at-point-functions 'ess-R-object-completion nil 'local)
     (add-hook 'completion-at-point-functions 'ess-filename-completion nil 'local)
-    (setq comint-input-sender 'inferior-R-input-sender)
+    (setq comint-input-sender 'inferior-ess-r-input-sender)
 
     (if gdbp
         (progn
@@ -421,7 +419,7 @@ Executed in process buffer."
      ;; "invisible(Sys.sleep(10))\n" nil (get-process ess-local-process-name) ;; test only
      (lambda (proc) (process-put proc 'packages-cached? t))))
 
-  (ess--R-load-ESSR)
+  (inferior-ess-r-load-ESSR)
 
   (when inferior-ess-language-start
     (ess-eval-linewise inferior-ess-language-start
@@ -444,9 +442,9 @@ Executed in process buffer."
 (defun R-mode  (&optional proc-name)
   "Major mode for editing R source.  See `ess-mode' for more help."
   (interactive "P")
-  (setq ess-customize-alist R-customize-alist)
+  (setq ess-customize-alist ess-r-customize-alist)
   ;;(setq imenu-generic-expression R-imenu-generic-expression)
-  (ess-mode R-customize-alist proc-name)
+  (ess-mode ess-r-customize-alist proc-name)
   ;; for emacs < 24
   (add-hook 'comint-dynamic-complete-functions 'ess-complete-object-name t 'local)
   ;; for emacs >= 24
@@ -475,20 +473,16 @@ Executed in process buffer."
   (ad-activate 'move-beginning-of-line)
   (ad-activate 'back-to-indentation)
   (ad-activate 'ess-eval-line-and-step)
-  (if ess-roxy-hide-show-p
+  (when ess-roxy-hide-show-p
     (ad-activate 'ess-indent-command))
-  (substitute-key-definition 'newline-and-indent
-                             'ess-newline-and-indent
-                             ess-mode-map global-map)
-
-  (substitute-key-definition 'indent-new-comment-line
-                             'ess-indent-new-comment-line
-                             ess-mode-map global-map)
 
   (run-hooks 'R-mode-hook))
 
 (fset 'r-mode 'R-mode)
 
+
+;;*;; Miscellaneous
+
 (defun ess-R-arch-2-bit (arch)
   "Translate R's architecture shortcuts/directory names to 'bits',
  i.e., \"32\" or \"64\" (for now)."
@@ -791,7 +785,7 @@ If BIN-RTERM-EXE is nil, then use \"bin/Rterm.exe\"."
 See `ess-noweb-mode' and `R-mode' for more help."
   (interactive)
   (require 'ess-noweb);; << probably someplace else
-  (setq ess--make-local-vars-permenent t)
+  (setq ess--make-local-vars-permanent t)
   (ess-noweb-mode 1); turn it on
   (ess-noweb-set-doc-mode 'latex-mode)
   (ess-noweb-set-code-mode 'R-mode)
@@ -812,7 +806,7 @@ See `ess-noweb-mode' and `R-mode' for more help."
 (defun R-transcript-mode ()
   "Does the right thing."
   (interactive)
-  (ess-transcript-mode R-customize-alist))
+  (ess-transcript-mode ess-r-customize-alist))
 
 (fset 'r-transcript-mode 'R-transcript-mode)
 
@@ -1006,7 +1000,339 @@ similar to `load-library' emacs function."
   '(eldoc-add-command "ess-smart-comma"))
 
 
+;;*;; Interaction with R
+
+;;;*;;; Evaluation
+
+(defun ess-r-arg (param value &optional wrap)
+  (let ((value (if wrap
+                   (concat "'" value "'")
+                 value)))
+    (concat ", " param " = " value)))
+
+(defun ess-r-build-args (visibly output namespace)
+  (let ((visibly (ess-r-arg "visibly" (if visibly "TRUE" "FALSE")))
+        (output (ess-r-arg "output" (if output "TRUE" "FALSE")))
+        (pkg (when namespace (ess-r-arg "package" namespace t)))
+        (verbose (when (and namespace
+                            ess-r-namespaced-load-verbose)
+                   (ess-r-arg "verbose" "TRUE"))))
+    (concat visibly output pkg verbose)))
+
+(ess-defmethod R ess-build-eval-command (string &optional visibly output file namespace)
+  (let* ((namespace (unless ess-debug-minor-mode
+                      (or namespace (ess-r-get-evaluation-env))))
+         (cmd (if namespace ".ess.ns_eval" ".ess.eval"))
+         (file (when file (ess-r-arg "file" file t)))
+         (args (ess-r-build-args visibly output namespace)))
+    (concat cmd "(\"" string "\"" args file ")\n")))
+
+(ess-defmethod R ess-build-load-command (file &optional visibly output namespace)
+  (let* ((namespace (or namespace (ess-r-get-evaluation-env)))
+         (cmd (if namespace ".ess.ns_source" ".ess.source"))
+         (args (ess-r-build-args visibly output namespace)))
+    (concat cmd "('" file "'" args ")\n")))
+
+(defun ess-r-build-eval-message (message)
+  (let ((env (cond ((ess-r-get-evaluation-env))
+                   (ess-debug-minor-mode
+                    (substring-no-properties ess-debug-indicator 1)))))
+    (if env
+        (format "[%s] %s" env message)
+      message)))
+
+(defvar-local ess-r-evaluation-env nil
+  "Environment into which code should be evaluated.
+When this variable is nil, code is evaluated in the current
+environment. Currently only packages can be set as evaluation
+environments. Use `ess-r-set-evaluation-env' to set this
+variable.")
+
+(defun ess-r-get-evaluation-env ()
+  "Get current evaluation env."
+  (or ess-r-evaluation-env
+      (and ess-current-process-name
+           (ess-get-process-variable 'ess-r-evaluation-env))))
+
+(defvar ess-r-prompt-for-attached-pkgs-only nil
+  "If nil provide completion for all installed R packages.
+If non-nil, only look for attached packages.")
+
+(defun ess-r-set-evaluation-env (&optional arg)
+  "Select a package namespace for evaluation of R code.
+
+Call interactively with a prefix argument to disable evaluation
+in a namespace.  When calling from a function, ARG can be a
+string giving the package to select, any other non-nil value to
+disable, or nil to prompt for a package.
+
+If `ess-r-prompt-for-attached-pkgs-only' is non-nil, prompt only for
+attached packages."
+  (interactive "P")
+  (let ((env (cond ((stringp arg) arg)
+                   ((null arg) (ess-r--select-package-name))
+                   (t "*none*"))))
+    (if (equal env "*none*")
+        (let ((cur-env (ess-r-get-evaluation-env)))
+          ;; fixme: does not work if env is set at process level
+          (setq ess-r-evaluation-env nil)
+          (delq 'ess-r--evaluation-env-mode-line ess--local-mode-line-process-indicator)
+          (message (format "Evaluation in %s disabled" (propertize cur-env 'face font-lock-function-name-face))))
+      (setq ess-r-evaluation-env env)
+      (add-to-list 'ess--local-mode-line-process-indicator 'ess-r--evaluation-env-mode-line t)
+      (message (format "Evaluating in %s" (propertize env 'face font-lock-function-name-face))))
+    (force-mode-line-update)))
+
+(defvar-local ess-r--evaluation-env-mode-line 
+  '(:eval (let ((env (ess-r-get-evaluation-env)))
+            (if env
+                (format " %s"
+                        (propertize  (if (equal env (car (ess-r-package-get-info)))
+                                         "pkg"
+                                       env)
+                                     'face 'mode-line-emphasis))
+              ""))))
+(put 'ess-r--evaluation-env-mode-line 'risky-local-variable t)
+
+(defvar ess-r-namespaced-load-verbose t
+  "Whether to display information on namespaced loading.
+
+When t, loading a file into a namespaced will output information
+about which objects are exported and which stay hidden in the
+namespace.")
+
+(defvar ess-r-namespaced-load-only-existing t
+  "Whether to load only objects already existing in a namespace.")
+
+(ess-defmethod R ess-load-file (file)
+  (cond
+   ;; Namespaced evaluation
+   ((ess-r-get-evaluation-env)
+    (ess-r-load-file-namespaced file))
+   ;; Evaluation into current env via .ess.source()
+   (t
+    (let ((command (ess-build-load-command file nil t)))
+      (ess-send-string (ess-get-process) command)))))
+
+(defun ess-r-load-file-namespaced (&optional file)
+  "Load FILE into a package namespace.
+
+This prompts for a package when no package is currently
+selected (see `ess-r-set-evaluation-env')."
+  (interactive)
+  (ess-force-buffer-current "R process to use: ")
+  (let* ((pkg-name (ess-r-get-evaluation-env))
+         (command (ess-build-load-command file nil t pkg-name)))
+    (ess-send-string (ess-get-process) command)))
+
+(ess-defmethod R ess-send-region (proc start end visibly message type)
+  (cond
+   ;; Namespaced evaluation
+   ((ess-r-get-evaluation-env)
+    (ess-r-send-region-namespaced proc start end visibly message))
+   ;; Evaluation into current env
+   (t
+    (ess-send-string proc (buffer-substring start end) visibly message type))))
+
+(defun ess-r-send-region-namespaced (proc beg end &optional visibly message)
+  "Ask for for the package and devSource region into it."
+  (let* ((pkg-name (or (ess-r-get-evaluation-env)
+                       (ess-r-set-evaluation-env))))
+    (message (ess-r-build-eval-message (or message "Eval region"))))
+  (ess-send-string proc (buffer-substring start end) visibly message))
+
+
+;;;*;;; Help
+
+(defun ess-r-namespaced-object-p (object)
+  (string-match "^[[:alnum:].]+::" object))
+
+(defun ess-r-build-help-command--qualified (object)
+  (when (ess-r-namespaced-object-p object)
+    (let* ((pkg-name (substring object (match-beginning 0) (- (match-end 0) 2)))
+           (object (concat "'" (substring object (match-end 0)) "'"))
+           (pkg (ess-r-arg "package" pkg-name t)))
+      (concat ".ess.help(" object pkg ")\n"))))
+
+(defun ess-r-build-help-command--get-package-dir (object dont-ask)
+  ;; Ugly hack to avoid tcl/tk dialogues
+  (let ((pkgs (ess-get-words-from-vector
+               (format "as.character(help('%s'))\n" object))))
+    (when (> (length pkgs) 1)
+      (if dont-ask
+          (car pkgs)
+        (ess-completing-read "Choose location" pkgs nil t)))))
+
+(defun ess-r-build-help-command--unqualified (object dont-ask)
+  (if (eq ess-help-type 'index)
+      ;; we are in index page, qualify with namespace
+      (ess-r-build-help-command--qualified (format "%s::%s" ess-help-object object))
+    (let ((pkg-dir (ess-r-build-help-command--get-package-dir object dont-ask))
+          (command (format inferior-ess-r-help-command object)))
+      (if pkg-dir
+          ;; Invoking `print.help_files_with_topic'
+          (format "do.call(structure, c('%s', attributes(%s)))\n" pkg-dir command)
+        command))))
+
+(defun ess-r-build-help-command (object &optional dont-ask)
+  (or (ess-r-build-help-command--qualified object)
+      (ess-r-build-help-command--unqualified object dont-ask)))
+
+(defconst inferior-ess-r--input-help (format "^ *help *(%s)" ess-help-arg-regexp))
+(defconst inferior-ess-r--input-?-help-regexp "^ *\\(?:\\(?1:[a-zA-Z ]*?\\?\\{1,2\\}\\) *\\(?2:.+\\)\\)")
+(defconst inferior-ess-r--page-regexp (format "^ *page *(%s)" ess-help-arg-regexp))
+
+(defvar ess-help-r--last-help-type nil
+  "Variable holding the last known help type. If it changes,
+we flush the cache.")
+
+(defun ess-help-r--check-last-help-type ()
+  (let ((help-type (ess-string-command "getOption('help_type')\n")))
+    (when (not (string= help-type ess-help-r--last-help-type))
+      (let ((help-buffers (ess-help-get-local-help-buffers)))
+        (mapc #'kill-buffer help-buffers))
+      (setq ess-help-r--last-help-type help-type))))
+
+(defun ess-help-r--process-help-input (proc string)
+  (let ((help-match (and (string-match inferior-ess-r--input-help string)
+                         (match-string 2 string)))
+        (help-?-match (and (string-match inferior-ess-r--input-?-help-regexp string)
+                           string))
+        (page-match   (and (string-match inferior-ess-r--page-regexp string)
+                           (match-string 2 string))))
+    (when (or help-match help-?-match page-match)
+      (ess-help-r--check-last-help-type)
+      (cond (help-match
+             (ess-display-help-on-object help-match)
+             (process-send-string proc "\n"))
+            (help-?-match
+             (ess-help-r--display-help-? proc string help-?-match)
+             (process-send-string proc "\n"))
+            (page-match
+             (switch-to-buffer-other-window
+              (ess-command (concat page-match "\n")
+                           (get-buffer-create (concat page-match ".rt"))))
+             (R-transcript-mode)
+             (process-send-string proc "\n")))
+      t)))
+
+(defun ess-help-r--display-help-? (proc string help-?-match)
+  (cond ((string-match "\\?\\?\\(.+\\)" help-?-match)
+         (ess--display-indexed-help-page (concat help-?-match "\n")
+                                         "^\\([^ \t\n]+::[^ \t\n]+\\)[ \t\n]+"
+                                         (format "*ess-apropos[%s](%s)*"
+                                                 ess-current-process-name (match-string 1 help-?-match))
+                                         'appropos))
+        ((string-match "^ *\\? *\\([^ \t]+\\)$" help-?-match)
+         (ess-display-help-on-object (match-string 1 help-?-match)))
+        ;; Anything else we send to process almost unchanged
+        (t
+         (let ((help-?-match (and (string-match inferior-ess-r--input-?-help-regexp string)
+                                  (format "%s%s" (match-string 1 string)
+                                          (ess-help-r--sanitize-topic (match-string 2 string))))))
+           (ess-display-help-on-object help-?-match "%s\n")))))
+
+(defun ess-help-r--sanitize-topic (string)
+  ;; Enclose help topics into `` to avoid ?while ?if etc hangs
+  (if (string-match "\\([^:]*:+\\)\\(.*\\)$" string) ; treat foo::bar corectly
+      (format "%s`%s`" (match-string 1 string) (match-string 2 string))
+    (format "`%s`" string)))
+
+;;;*;;; Utils for inferior R process
+
+(defun inferior-ess-r-input-sender (proc string)
+  (save-current-buffer
+    (or (ess-help-r--process-help-input proc string)
+        (inferior-ess-input-sender proc string))))
+
+(defun inferior-ess-r-load-ESSR ()
+  "Load/INSTALL/Update ESSR."
+  (let* ((pkg-dir (expand-file-name "ESSR" ess-etc-directory))
+         (src-dir (expand-file-name "R" pkg-dir)))
+
+    (if (not (or (and (boundp 'ess-remote) ess-remote)
+                 (file-remote-p (ess-get-process-variable 'default-directory))))
+        (inferior-ess-r-load-ESSR--local pkg-dir src-dir)
+      (inferior-ess-r-load-ESSR--remote pkg-dir src-dir))))
+
+(defun inferior-ess-r-load-ESSR--local (pkg-dir src-dir)
+  (let ((cmd (format "local({
+                          source('%s/.load.R', local=TRUE) #define load.ESSR
+                          load.ESSR('%s')
+                      })\n"
+                     src-dir src-dir)))
+    (ess-write-to-dribble-buffer (format "load-ESSR cmd:\n%s\n" cmd))
+    (with-current-buffer (ess-command cmd)
+      (let ((msg (buffer-string)))
+        (when (> (length msg) 1)
+          (message (format "load ESSR: %s" msg)))))))
+
+(defun inferior-ess-r-load-ESSR--remote (pkg-dir src-dir)
+  (let* ((verfile (expand-file-name "VERSION" pkg-dir))
+         (loadremote (expand-file-name "LOADREMOTE" pkg-dir))
+         (version (if (file-exists-p verfile)
+                      (with-temp-buffer
+                        (insert-file-contents verfile)
+                        (buffer-string))
+                    (error "Cannot find ESSR source code")))
+         (r-load-code (with-temp-buffer
+                        (insert-file-contents loadremote)
+                        (buffer-string))))
+    (ess-write-to-dribble-buffer (format "version file: %s\nloadremote file: %s\n"
+                                         verfile loadremote))
+    (unless (ess-boolean-command (format r-load-code version) nil 0.1)
+      (let ((errmsg (with-current-buffer " *ess-command-output*" (buffer-string)))
+            (files (directory-files src-dir t "\\.R$")))
+        (ess-write-to-dribble-buffer (format "error loading ESSR.rda: \n%s\n" errmsg))
+        ;; should not happen, unless extrem conditions (ancient R or failed download))
+        (message "Failed to download ESSR.rda (see *ESS* buffer). Injecting ESSR code from local machine")
+        (ess-command (format ".ess.ESSRversion <- '%s'\n" version)) ; cannot do this at R level
+        (mapc #'ess--inject-code-from-file files)))))
+
+(ess-defmethod R ess-quit (&optional no-save)
+  (let (cmd
+        ;;Q     response
+        (sprocess (ess-get-process ess-current-process-name)))
+    (if (not sprocess) (error "No ESS process running"))
+    ;;Q (setq response (completing-read "Save workspace image? "
+    ;;Q                                 '( ( "yes".1) ("no" . 1) ("cancel" . 1))
+    ;;Q                                 nil t))
+    ;;Q (if (string-equal response "")
+    ;;Q (setq response "default")); which will ask again (in most situations)
+    ;;Q (unless (string-equal response "cancel")
+    (ess-cleanup)
+    ;;Q   (setq cmd (format "q(\"%s\")\n" response))
+    (setq cmd (format "base::q('%s')\n" (if no-save "no" "default")))
+    (goto-char (marker-position (process-mark sprocess)))
+    (process-send-string sprocess cmd)))
+
+(defcustom inferior-ess-r-reload-hook nil
+  "Hook run when reloading the R inferior buffer."
+  :type 'hook
+  :group 'ess-R)
+
+(defun inferior-ess-r-reload (&optional start-args)
+  "Reload R and the currently activated developer package, if any."
+  (interactive)
+  (ess-force-buffer-current)
+  (let ((pkg-info ess-r-package-info)
+        (r-proc (ess-get-process)))
+    (with-ess-process-buffer nil
+      (ess-quit 'no-save)
+      (while (memq (process-status r-proc) '(run busy))
+        (accept-process-output r-proc 0.002))
+      (kill-buffer)
+      (R start-args)
+      (when pkg-info
+        (setq-local ess-r-package-info pkg-info)
+        (ess-r-devtools-load-package))
+      (run-hooks 'inferior-ess-r-reload-hook))))
+
+
+;;*;; Editing Tools
+
 ;;;*;;; Indentation Engine
+
 ;; Written by Lionel Henry in mid 2015
 
 (defun ess-r-indent-line ()
@@ -1516,7 +1842,7 @@ otherwise nil."
     (let* ((start-line (line-number-at-pos))
            (prev-pos 0)
            (cascade (eq (ess-offset-type 'continued) 'cascade))
-           (climbed (progn (ess-climb-continuations cascade)))
+           (climbed (ess-climb-continuations cascade))
            max-col)
       (when climbed
         (cond
@@ -1573,7 +1899,7 @@ otherwise nil."
              (looking-at (concat match "("))
              (current-column))))))
 
-
+
 ;;;*;;; Call filling engine
 
 ;; Unroll arguments to a single line until closing marker is found.
